<?php
require_once 'PHPUnit/Framework.php';
require_once 'Maarch_CLITools/ArgsParser.php';

class ArgsParserForTest extends ArgsParser{
    function __get($key){
        if (property_exists($this, $key)){
            return $this->$key;
        }
        return null;
    }
}


class ArgsParserTest extends PHPUnit_Framework_TestCase{
    public function setUp(){
        $this->parser = new ArgsParserForTest();
        $this->assertTrue($this->parser instanceof ArgsParser);
    }
    public function tearDown(){
        unset($this->parser);
        $this->assertNull($this->parser);
    }


    public function testInitialisation(){
        $this->assertObjectHasAttribute("options", $this->parser);
        $this->assertAttributeSame(Array("positional"=>Array()), "options", $this->parser);

        $this->assertObjectHasAttribute("mapping", $this->parser);
        $this->assertAttributeSame(Array(), "mapping", $this->parser);

        $this->assertObjectHasAttribute("args", $this->parser);
        $this->assertAttributeSame(Array(), "args", $this->parser);

        $this->assertObjectHasAttribute("positionals", $this->parser);
        $this->assertAttributeSame(Array(), "positionals", $this->parser);

        $this->assertObjectHasAttribute("name", $this->parser);
        $this->assertAttributeSame("__main__", "name", $this->parser);

    }

    function testAddSwitchNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
    }

    function testAddSwitchOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", "q");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["q"]);
    }

    function testAddSwitchOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["quiet"]);
    }

    function testAddSwitchShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["quiet"]);
    }

    function testAddSwitchOnlyCatalog(){
        $this->parser->add_arg("quiet", "switch", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
    }

    function testAddSwitchAll(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->mapping));
    }

    function testAddArgumentNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
    }

    function testAddArgumentOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", "q");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["q"]);
    }

    function testAddArgumentOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["quiet"]);
    }

    function testAddArgumentShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->mapping);
        $this->assertSame("quiet", $this->parser->mapping["quiet"]);
    }

    function testAddArgumentOnlyCatalog(){
        $this->parser->add_arg("quiet", "argument", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
    }

    function testAddArgumentAll(){
        $this->parser->add_arg("quiet", "argument", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->mapping));
    }

    function testAddPositionalNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddPositionalOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", "q");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddPositionalOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddPositionalShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddPositionalOnlyCatalog(){
        $this->parser->add_arg("quiet", "positional", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddPositionalAll(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->args);
        $this->assertSame("quiet", $this->parser->args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->mapping));
        $this->assertSame(Array("quiet"), $this->parser->positionals);
    }

    function testAddCommandMinimal(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array(), $subparser);
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->args);
        $this->assertSame('sub', $this->parser->args['sub']['name']);
        $this->assertSame('command', $this->parser->args['sub']['type']);
        $this->assertSame('', $this->parser->args['sub']['help']);
        $this->assertSame(Array(), $this->parser->args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->mapping);
        $this->assertSame("sub", $this->parser->mapping["sub"]);
    }

    function testAddCommandWithAliases(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array("sub2", "sub3"), $subparser, "my help");
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->args);
        $this->assertSame('sub', $this->parser->args['sub']['name']);
        $this->assertSame('command', $this->parser->args['sub']['type']);
        $this->assertSame("my help", $this->parser->args['sub']['help']);
        $this->assertSame(Array("sub2", "sub3"), $this->parser->args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->mapping);
        $this->assertSame("sub", $this->parser->mapping["sub"]);
        $this->assertArrayHasKey('sub2', $this->parser->mapping);
        $this->assertSame("sub", $this->parser->mapping["sub2"]);
        $this->assertArrayHasKey('sub3', $this->parser->mapping);
        $this->assertSame("sub", $this->parser->mapping["sub3"]);
    }

}